; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			
; SLEEP+INT0+INT1 (externer Interrupt) konfigurieren						
;																			
; MCUCR => SE SM2 SM1 SM0 ISC11 ISC10 ISC01 ISC00 							
; SM2 SM1 SM0 Sleep Mode													
; 0   0   0   Idle															
; 0   0   1   ADC Noise Reduction											
; 0   1   0   Power-down													
;*0   1   1   Power-save													
; 1   0   0   Reserved														
; 1   0   1   Reserved														
; 1   1   0   Standby(1)													
;																			
;ISC11 ISC10 Description													
; 0 	0 	The low level of INT1 generates an interrupt request.			
; 0 	1 	Any logical change on INT1 generates an interrupt request.		
;*1 	0 	The falling edge of INT1 generates an interrupt request.		
; 1 	1 	The rising edge of INT1 generates an interrupt request.			
;																			
;ISC01 ISC00 Description													
; 0 	0 	The low level of INT0 generates an interrupt request.			
; 0 	1 	Any logical change on INT0 generates an interrupt request.		
;*1 	0 	The falling edge of INT0 generates an interrupt request.		
; 1 	1 	The rising  edge of INT0 generates an interrupt request.		
;																			
SLEEP_EXT_INTERRUPT_INITIALISIEREN:
;	ldi temp,(1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0)|(0<<ISC11)|(0<<ISC10)|(0<<ISC01)|(0<<ISC00) ; Idle 			  2mA
	ldi temp,(1<<SE)|(0<<SM2)|(1<<SM1)|(1<<SM0)|(0<<ISC11)|(0<<ISC10)|(0<<ISC01)|(0<<ISC00) ; Power save	0,2mA nur bei ASSR
	out MCUCR,temp

;	ldi temp, (0<<INT1)|(1<<INT0)|(0<<IVSEL)|(0<<IVCE)   
;   out GICR, temp 

;	cbi DDRD, 2
;	sbi PORTD,2
ret
;###################################################################
;###################################################################
;###################################################################
SLEEP_1ms:
	WDR
	SLEEP
	IN temp1,TCNT2
SLEEP_1ms_s:
	IN temp,TCNT2
	cp temp1,temp
	breq SLEEP_1ms_s
ret
;###################################################################
SLEEP_2ms:
	rcall SLEEP_1ms
	rjmp  SLEEP_1ms
;###################################################################
SLEEP_5ms:
	rcall SLEEP_2ms
	rcall SLEEP_2ms
	rjmp  SLEEP_1ms
;###################################################################
SLEEP_10ms:
	rcall SLEEP_5ms
	rjmp  SLEEP_5ms
;###################################################################
SLEEP_100ms:
	ldi temp9,10
SLEEP_100ms_s:
	rcall SLEEP_10ms
	dec temp9
	brne SLEEP_100ms_s
ret
;###################################################################
SLEEP_1s:
	ldi temp9,100
SLEEP_1s_s:
	rcall SLEEP_10ms
	dec temp9
	brne SLEEP_1s_s
ret
;###################################################################
;###################################################################
;###################################################################
TIMER_AKTIVIEREN:
 	ldi temp,(1<<TOIE2)|(1<<TOIE1)|(1<<TOIE0)
	out TIMSK,temp          
ret
;###################################################################
;###################################################################
;###################################################################
; CS22 CS21 CS20 Description										
;	0	0 	0 	No clock source (Timer/Counter stopped).			
;	0 	0 	1 	clkT2S/		(No prescaling)							
; *	0 	1 	0 	clkT2S/8 	(From prescaler)						
;	0 	1 	1 	clkT2S/32 	(From prescaler)						
; 	1 	0 	0 	clkT2S/64 	(From prescaler)						
;	1 	0 	1 	clkT2S/128 	(From prescaler)						
;   1 	1 	0 	clkT2S/256 	(From prescaler)						
;	1 	1 	1 	clkT2S/1024 (From prescaler)						
TIMER2_INITIALISIEREN_1MHz_1ms:
	ldi temp,(0<<CS22)|(1<<CS21) |(0<<CS20) 
	out TCCR2,temp  

	ldi temp,(256-125)		; (2 hoch 8  minus...) => 1ms 	
	mov TIMERZAHL,temp

	out TCNT2,TIMERZAHL		; Timerwert			
ret
;###################################################################
;###################################################################
;###################################################################
; CS22 CS21 CS20 Description										
;	0	0 	0 	No clock source (Timer/Counter stopped).			
;	0 	0 	1 	clkT2S/		(No prescaling)							
; *	0 	1 	0 	clkT2S/8 	(From prescaler)						
;	0 	1 	1 	clkT2S/32 	(From prescaler)						
; 	1 	0 	0 	clkT2S/64 	(From prescaler)						
;	1 	0 	1 	clkT2S/128 	(From prescaler)						
;   1 	1 	0 	clkT2S/256 	(From prescaler)						
;	1 	1 	1 	clkT2S/1024 (From prescaler)						
TIMER2_INITIALISIEREN_ASYNCRON_1ms:
.equ TIMERWERT2=(256-4)
	ldi temp,(1<<AS2)|(0<<TCN2UB)|(0<<OCR2UB)|(0<<TCR2UB)
	out ASSR ,temp

	ldi temp,(0<<CS22)|(1<<CS21) |(0<<CS20) 
	out TCCR2,temp  

	ldi temp,TIMERWERT2
	mov TIMERZAHL,temp
	out TCNT2,TIMERZAHL				; Timerwert			
ret
;###################################################################
;###################################################################
;###################################################################
; nach jeder 1000-el Sekunde wird dieser Interrupt aufgerufen		
TIM2_OVF: 
	sbi (WIND_PORT),(WIND_PIN_NR)	; Pull-Up on		
	out TCNT2,TIMERZAHL				; Timerwert			
	push TEMP		    			; Sichern von "TEMP" im Stack			
	in	 TEMP,SREG					; Einlesen des SREG 					
	push TEMP	    				; Schreiben von  SREG  im Stack (KOPIE)	
	LED_GELB_OFF					; LED				
;-------------------------------------------------------------------
; WIND-SAMPLING														
	SEC 							; Set Carry				
	sbic (WIND_PIN),(WIND_PIN_NR)	; springe, wenn PIN...	
	CLC 							; Clear Carry			
	cbi (WIND_PORT),(WIND_PIN_NR)	; Pull-Up off			
	ROL ROHDATA						; links ber Carry		
	mov temp,ROHDATA
	andi temp,0b00001111			; Bit-Muster			
	mov ROHDATA,temp
	ldi  temp,0b00000011			; Bit-Muster (2xL 2xH)	
	cp ROHDATA,temp
	brne TIM_w1
	inc IMPULSE						; +1					
	LED_GELB_ON						; LED					
TIM_w1:
;-------------------------------------------------------------------
; Sekundenvorteiler													
	sub ssVT_L,EINS					; VT-1	
	sbc ssVT_H,NULL
	tst ssVT_L						; NULL?	
	brne TIM_FERTIG
	tst ssVT_H						; NULL?	
	brne TIM_FERTIG
;-------------------------------------------------------------------
; Sekundentakt														
	ldi temp,LOW (1000)				; VT	
	mov ssVT_L, temp
	ldi temp,HIGH(1000)				; VT	
	mov ssVT_H, temp
;-------------------------------------------------------------------
	ldi temp,(SCHWELLE0)			; WIND-Min ?
	cp IMPULSE,temp
	brsh TIM_w2
	clr IMPULSE
TIM_w2:
	mov FREQUENZ,IMPULSE			; Frequenz	
	clr IMPULSE
	STS(adr_FREIGABE),EINS
;-------------------------------------------------------------------
TIM_FERTIG:
	pop	TEMP						; LESEN von SREG vom STACK (KOPIE)		
	out	SREG,TEMP					; Wiederherstellen von SREG				
	pop	TEMP						; Wiederherstellen von "TEMP" 	
reti    
;###################################################################
;###################################################################
;###################################################################
